package org.test;

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import org.drools.core.RuleBaseConfiguration;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieRepository;
import org.kie.api.conf.EventProcessingOption;
import org.kie.api.io.Resource;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.KieSessionConfiguration;
import org.kie.internal.KnowledgeBaseFactory;

public class Drools6Test {

	private static final String RULE_FILENAME = "defaultKBase/Test.drl";

	@Test
	public void testAfter_ok() throws InterruptedException, IOException {
		KieSession kSession = initSession(false);
		kSession.insert(new SimpleFact("id1", "code1"));
		kSession.fireAllRules();
		assertEquals("1 facts in session", 1, kSession.getFactCount());
		Thread.sleep(3000);
		assertEquals("2 facts in session", 2, kSession.getFactCount());
	}

	@Test
	public void testAfter_error() throws InterruptedException, IOException {
		KieSession kSession = initSession(false);
		kSession.insert(new SimpleFact("id1", "code2"));
		kSession.fireAllRules();
		assertEquals("1 facts in session", 1, kSession.getFactCount());
		Thread.sleep(3000);
		assertEquals("2 facts in session", 2, kSession.getFactCount());
	}

	protected KieSession initSession(boolean phreakMode) {
		KieServices ks = KieServices.Factory.get();
		KieRepository kr = ks.getRepository();
		KieFileSystem kfs = ks.newKieFileSystem();
		Resource res = ks.getResources().newClassPathResource(RULE_FILENAME);
		kfs.write(res);

		KieBuilder kb = ks.newKieBuilder(kfs);
		kb.buildAll();
		if (kb.getResults().hasMessages()) {
			System.err.println("!!! Problem: " + kb.getResults().toString());
		}

		KieContainer kContainer = ks.newKieContainer(kr.getDefaultReleaseId());

		RuleBaseConfiguration kBaseConf = new RuleBaseConfiguration();
		kBaseConf.setEventProcessingMode(EventProcessingOption.STREAM);
		kBaseConf.setPhreakEnabled(phreakMode);

		KieBase kbase = kContainer.newKieBase(kBaseConf);
		KieSessionConfiguration ksessionConfig = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
		KieSession ksession = kbase.newKieSession(ksessionConfig, null);

		MyAgendaListener agendaListener = new MyAgendaListener();
		ksession.addEventListener(agendaListener);

		return ksession;
	}
}

